@@ -9,7 +9,7 @@
 #include <string>
 #include <vector>
 
-#include "util/test.h"
+#include "gtest/gtest.h"
 #include "util/logging.h"
 #include "re2/regexp.h"
 
@@ -23,7 +23,7 @@ TEST(Regexp, BigRef) {
     re->Incref();
   for (int i = 0; i < 100000; i++)
     re->Decref();
-  CHECK_EQ(re->Ref(), 1);
+  ASSERT_EQ(re->Ref(), 1);
   re->Decref();
 }
 
@@ -35,12 +35,12 @@ TEST(Regexp, BigConcat) {
   std::vector<Regexp*> v(90000, x);  // ToString bails out at 100000
   for (size_t i = 0; i < v.size(); i++)
     x->Incref();
-  CHECK_EQ(x->Ref(), 1 + static_cast<int>(v.size())) << x->Ref();
+  ASSERT_EQ(x->Ref(), 1 + static_cast<int>(v.size())) << x->Ref();
   Regexp* re = Regexp::Concat(v.data(), static_cast<int>(v.size()),
                               Regexp::NoParseFlags);
-  CHECK_EQ(re->ToString(), string(v.size(), 'x'));
+  ASSERT_EQ(re->ToString(), std::string(v.size(), 'x'));
   re->Decref();
-  CHECK_EQ(x->Ref(), 1) << x->Ref();
+  ASSERT_EQ(x->Ref(), 1) << x->Ref();
   x->Decref();
 }
 
@@ -51,11 +51,11 @@ TEST(Regexp, NamedCaptures) {
       "(?P<g1>a+)|(e)(?P<g2>w*)+(?P<g1>b+)", Regexp::PerlX, &status);
   EXPECT_TRUE(status.ok());
   EXPECT_EQ(4, x->NumCaptures());
-  const std::map<string, int>* have = x->NamedCaptures();
+  const std::map<std::string, int>* have = x->NamedCaptures();
   EXPECT_TRUE(have != NULL);
   EXPECT_EQ(2, have->size());  // there are only two named groups in
                                // the regexp: 'g1' and 'g2'.
-  std::map<string, int> want;
+  std::map<std::string, int> want;
   want["g1"] = 1;
   want["g2"] = 3;
   EXPECT_EQ(want, *have);
@@ -70,10 +70,10 @@ TEST(Regexp, CaptureNames) {
       "(?P<g1>a+)|(e)(?P<g2>w*)+(?P<g1>b+)", Regexp::PerlX, &status);
   EXPECT_TRUE(status.ok());
   EXPECT_EQ(4, x->NumCaptures());
-  const std::map<int, string>* have = x->CaptureNames();
+  const std::map<int, std::string>* have = x->CaptureNames();
   EXPECT_TRUE(have != NULL);
   EXPECT_EQ(3, have->size());
-  std::map<int, string> want;
+  std::map<int, std::string> want;
   want[1] = "g1";
   want[3] = "g2";
   want[4] = "g1";
